use sodiumoxide::crypto::{pwhash, secretbox};


pub fn save(data: &str, passphrase: &str) {
    let salt = pwhash::Salt([0; 32]); // yes yes, just for now
    let mut kbuf = [0; secretbox::KEYBYTES];
    pwhash::derive_key_interactive(&mut kbuf, passphrase.as_bytes(), &salt)
        .expect("Couldn't compute password hash. Are you out of memory?");
    let key = secretbox::Key(kbuf);
    let nonce = secretbox::Nonce([0; 24]); // we don't care about e.g. replay attacks so this might be safe?
    let encrypted = secretbox::seal(data.as_bytes(), &nonce, &key);

    //todo: store in a database, along with salt, nonce, and hash parameters
    std::fs::write("credentials.enc", &encrypted).expect("Failed to write file.");

    //todo: key is automatically zeroed, but we should use 'zeroize' or something to zero out passphrase and data
}


pub fn load(passphrase: &str) -> String {
    let salt = pwhash::Salt([0; 32]);
    let mut kbuf = [0; secretbox::KEYBYTES];
    pwhash::derive_key_interactive(&mut kbuf, passphrase.as_bytes(), &salt)
        .expect("Couldn't compute password hash. Are you out of memory?");
    let key = secretbox::Key(kbuf);
    let nonce = secretbox::Nonce([0; 24]);

    let encrypted = std::fs::read("credentials.enc").expect("Failed to read file.");
    let decrypted = secretbox::open(&encrypted, &nonce, &key).expect("Failed to decrypt.");
    String::from_utf8(decrypted).expect("Invalid utf-8")
}
